@@ -17,8 +17,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 """Activity Views.
-
-$Id$
 """
 from decimal import Decimal, InvalidOperation
 
@@ -27,8 +25,10 @@ from StringIO import StringIO
 
 from zope.container.interfaces import INameChooser
 from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
+from zope.i18n import translate
 from zope.interface import implements
 from zope.publisher.browser import BrowserView
+import zope.schema
 from zope.security.checker import canWrite
 from zope.security.interfaces import Unauthorized
 from zope.traversing.browser.absoluteurl import absoluteURL
@@ -46,6 +46,8 @@ from z3c.form.interfaces import DISPLAY_MODE
 
 from schooltool.app.interfaces import ISchoolToolApplication
 from schooltool.basicperson.interfaces import IDemographics
+from schooltool.common.inlinept import InheritTemplate
+from schooltool.common.inlinept import InlineViewPageTemplate
 from schooltool.course.interfaces import ISection, IInstructor
 from schooltool.export import export
 from schooltool.gradebook import GradebookMessage as _
@@ -53,12 +55,15 @@ from schooltool.gradebook import interfaces
 from schooltool.gradebook.activity import createSourceString, getSourceObj
 from schooltool.gradebook.activity import Activity, LinkedColumnActivity
 from schooltool.gradebook.activity import LinkedActivity, Activities
-from schooltool.gradebook.category import getCategories
+from schooltool.gradebook.activity import Worksheet
 from schooltool.person.interfaces import IPerson
 from schooltool.gradebook.browser.gradebook import LinkedActivityGradesUpdater
 from schooltool.requirement.interfaces import IRangedValuesScoreSystem
 from schooltool.requirement.scoresystem import RangedValuesScoreSystem
 from schooltool.term.interfaces import ITerm, IDateManager
+from schooltool.skin import flourish
+
+SUMMARY_TITLE = _('Summary')
 
 
 class ILinkedActivityFields(interface.Interface):
@@ -131,6 +136,110 @@ class ActivitiesView(object):
                     self.context.changePosition(name, new_pos-1)
 
 
+class FlourishWorksheetsView(flourish.page.Page):
+    """A flourish view of the gradebook's worksheets."""
+
+    @property
+    def title(self):
+        return self.context.__parent__.title
+
+    def positions(self):
+        return range(1, len(self.context.values())+1)
+
+    @property
+    def worksheets(self):
+        """Get  a list of all worksheets."""
+        pos, hidden, not_hidden = 1, [], []
+        activities = removeSecurityProxy(self.context)
+        for worksheet in super(Activities, activities).values():
+            result = {
+               'name': getName(worksheet),
+               'title': worksheet.title,
+               'url': absoluteURL(worksheet, self.request) + '/edit.html',
+               'pos': pos,
+               'checked': worksheet.hidden and 'checked' or '',
+               'deployed': worksheet.deployed,
+               }
+            if worksheet.hidden:
+                if not worksheet.deployed:
+                    hidden.append(result)
+            else:
+                not_hidden.append(result)
+                pos += 1
+        return not_hidden + hidden
+
+    def nextSummaryTitle(self):
+        index = 1
+        next = SUMMARY_TITLE
+        while True:
+            for worksheet in self.context.values():
+                if worksheet.title == next:
+                    break
+            else:
+                break
+            index += 1
+            next = SUMMARY_TITLE + str(index)
+        return next
+
+    def summaryFound(self):
+        return self.nextSummaryTitle() != SUMMARY_TITLE
+
+    def addSummary(self):
+        overwrite = self.request.get('overwrite', '') == 'on'
+        if overwrite:
+            currentWorksheets = []
+            for worksheet in self.context.values():
+                if worksheet.deployed:
+                    continue
+                if worksheet.title == SUMMARY_TITLE:
+                    while len(worksheet.values()):
+                        del worksheet[worksheet.values()[0].__name__]
+                    summary = worksheet
+                else:
+                    currentWorksheets.append(worksheet)
+            next = SUMMARY_TITLE
+        else:
+            next = self.nextSummaryTitle()
+            currentWorksheets = [worksheet for worksheet in self.context.values()
+                                 if not worksheet.deployed]
+            summary = Worksheet(next)
+            chooser = INameChooser(self.context)
+            name = chooser.chooseName('', summary)
+            self.context[name] = summary
+
+        for worksheet in currentWorksheets:
+            if worksheet.title.startswith(SUMMARY_TITLE):
+                continue
+            activity = LinkedColumnActivity(worksheet.title, u'assignment',
+                '', createSourceString(worksheet))
+            chooser = INameChooser(summary)
+            name = chooser.chooseName('', activity)
+            summary[name] = activity
+
+    def update(self):
+        if 'ADD_SUMMARY' in self.request:
+            self.addSummary()
+
+        elif 'form-submitted' in self.request:
+            old_pos = 0
+            for worksheet in self.context.values():
+                old_pos += 1
+                name = getName(worksheet)
+                if 'pos.'+name not in self.request:
+                    continue
+                new_pos = int(self.request['pos.'+name])
+                if new_pos != old_pos:
+                    self.context.changePosition(name, new_pos-1)
+
+            hidden = self.request.get('hidden', [])
+            activities = removeSecurityProxy(self.context)
+            for worksheet in super(Activities, activities).values():
+                if worksheet.hidden and not worksheet.__name__ in hidden:
+                    worksheet.hidden = False
+                if not worksheet.hidden and worksheet.__name__ in hidden:
+                    worksheet.hidden = True
+
+
 class UnhideWorksheetsView(object):
     """A view for unhiding woksheets."""
 
@@ -215,6 +324,80 @@ class ActivityAddView(z3cform.AddForm):
         return absoluteURL(self.context, self.request)
 
 
+class IActivityForm(interface.Interface):
+    '''An interface used to build flourish activity forms'''
+
+    title = zope.schema.TextLine(
+        title=_(u"Title"),
+        description=u'',
+        required=True)
+
+    label = zope.schema.TextLine(
+        title=_(u"Column Label"),
+        description=_("Limit to 5 characters or less."),
+        required=False)
+
+    due_date = zope.schema.Date(
+        title=_("Due Date"),
+        required=True)
+
+    description = zope.schema.Text(
+        title=_("Description"),
+        required=False)
+
+    category = zope.schema.Choice(
+        title=_("Category"),
+        description=_("Categories can be used to weigh different types of "
+                      "activites differently when calculating averages.  The "
+                      "list of categories can be set schoolwide set by your "
+                      "system administrator."),
+        vocabulary="schooltool.gradebook.category-vocabulary",
+        required=True)
+
+    max = zope.schema.Int(
+        title=_(u'Full Credit Score'),
+        description=_("This value must be an integer.  You may award extra "
+                      "credit above this value."),
+        required=True,
+        default=100)
+
+    min = zope.schema.Int(
+        title=_(u'Minimum Score'),
+        description=_('This value must be an integer.'),
+        required=True,
+        default=0)
+
+
+class SimpleFormAdapter(object):
+
+    def __init__(self, context):
+        self.__dict__['context'] = removeSecurityProxy(context)
+
+    def __setattr__(self, name, value):
+        setattr(self.context, name, value)
+
+    def __getattr__(self, name):
+        return getattr(self.context, name)
+
+
+class FlourishActivityAddView(flourish.form.AddForm, ActivityAddView):
+
+    template = InheritTemplate(flourish.page.Page.template)
+    label = None
+    legend = _('Activity Details')
+
+    fields = field.Fields(IActivityForm)
+
+    @button.buttonAndHandler(_('Submit'), name='add')
+    def handleAdd(self, action):
+        super(FlourishActivityAddView, self).handleAdd.func(self, action)
+
+    @button.buttonAndHandler(_("Cancel"))
+    def handle_cancel_action(self, action):
+        super(FlourishActivityAddView, self).handle_cancel_action.func(self,
+            action)
+
+
 class LinkedActivityAddView(z3cform.AddForm):
     """A view for adding a linked activity."""
 
@@ -271,6 +454,47 @@ class LinkedActivityAddView(z3cform.AddForm):
         LinkedActivityGradesUpdater().update(linked_activity, self.request)
 
 
+class ILinkedActivityExternalActivity(interface.Interface):
+
+    external_activity = schema.Choice(
+        title=_(u"External Score Source"),
+        vocabulary="schooltool.gradebook.external_activities",
+        required=True)
+
+
+class ILinkedActivityForm(IActivityForm):
+    '''An interface used to build flourish external activity forms'''
+
+    points = zope.schema.Int(
+        title=_(u"Full Credit Score"),
+        description=_("The point value of this activity will be calculated as "
+                      "the full credit score multiplied by the percentage "
+                      "value of the external score."),
+        min=0,
+        required=True)
+
+
+class FlourishLinkedActivityAddView(flourish.form.AddForm,
+                                    LinkedActivityAddView):
+
+    template = InheritTemplate(flourish.page.Page.template)
+    label = None
+    legend = _('External Score Details')
+
+    fields = field.Fields(ILinkedActivityExternalActivity, ILinkedActivityForm)
+    fields = fields.select("external_activity", "label", "due_date",
+                           "category", "points")
+
+    @button.buttonAndHandler(_('Submit'), name='add')
+    def handleAdd(self, action):
+        super(FlourishLinkedActivityAddView, self).handleAdd.func(self, action)
+
+    @button.buttonAndHandler(_("Cancel"))
+    def handle_cancel_action(self, action):
+        super(FlourishLinkedActivityAddView, self).handle_cancel_action.func(
+            self, action)
+
+
 class ActivityEditView(z3cform.EditForm):
     """Edit form for basic person."""
     z3cform.extends(z3cform.EditForm)
@@ -299,10 +523,35 @@ class ActivityEditView(z3cform.EditForm):
                  mapping={'fullname': self.context.title})
 
     def nextURL(self):
+        next = self.request.get('nexturl')
+        if next:
+            return next
         worksheet = self.context.__parent__
         return absoluteURL(worksheet, self.request) + '/manage.html'
 
 
+class FlourishActivityEditView(flourish.form.Form,
+                               ActivityEditView):
+
+    template = InheritTemplate(flourish.page.Page.template)
+    label = None
+    legend = _('Activity Details')
+
+    fields = field.Fields(IActivityForm).omit('max', 'min')
+
+    @button.buttonAndHandler(_('Submit'), name='apply')
+    def handleApply(self, action):
+        super(FlourishActivityEditView, self).handleApply.func(self, action)
+        # XXX: hacky sucessful submit check
+        if (self.status == self.successMessage or
+            self.status == self.noChangesMessage):
+            self.request.response.redirect(self.nextURL())
+
+    @button.buttonAndHandler(_("Cancel"))
+    def handle_cancel_action(self, action):
+        self.request.response.redirect(self.nextURL())
+
+
 class LinkedActivityEditView(z3cform.EditForm):
     """Edit form for linked activity."""
 
@@ -334,6 +583,32 @@ class LinkedActivityEditView(z3cform.EditForm):
         return absoluteURL(worksheet, self.request)
 
 
+class FlourishLinkedActivityEditView(flourish.form.Form,
+                                     LinkedActivityEditView):
+
+    template = InheritTemplate(flourish.page.Page.template)
+    label = None
+    legend = _('External Score Details')
+
+    fields = field.Fields(ILinkedActivityExternalActivity, mode=DISPLAY_MODE)
+    fields += field.Fields(ILinkedActivityForm)
+    fields = fields.select("external_activity", "title", 'label',
+                           'due_date', "description", "category",
+                           "points")
+
+    @button.buttonAndHandler(_('Submit'), name='apply')
+    def handleApply(self, action):
+        super(FlourishLinkedActivityEditView, self).handleApply.func(self, action)
+        # XXX: hacky sucessful submit check
+        if (self.status == self.successMessage or
+            self.status == self.noChangesMessage):
+            self.request.response.redirect(self.nextURL())
+
+    @button.buttonAndHandler(_("Cancel"))
+    def handle_cancel_action(self, action):
+        self.request.response.redirect(self.nextURL())
+
+
 class WeightCategoriesView(object):
     """A view for providing category weights for the worksheet context."""
 
@@ -347,12 +622,12 @@ class WeightCategoriesView(object):
 
     def update(self):
         self.message = ''
-        categories = getCategories(ISchoolToolApplication(None))
+        categories = interfaces.ICategoryContainer(ISchoolToolApplication(None))
         newValues = {}
         if 'CANCEL' in self.request:
             self.request.response.redirect(self.nextURL())
         elif 'UPDATE_SUBMIT' in self.request:
-            for category in sorted(categories.getKeys()):
+            for category in sorted(categories.keys()):
                 if category in self.request and self.request[category].strip():
                     value = self.request[category].strip()
                     try:
@@ -381,11 +656,10 @@ class WeightCategoriesView(object):
                     self.request.response.redirect(self.nextURL())
 
     def rows(self):
-        language = 'en' # XXX this need to be dynamic
         weights = self.context.getCategoryWeights()
-        categories = getCategories(ISchoolToolApplication(None))
+        categories = interfaces.ICategoryContainer(ISchoolToolApplication(None))
         result = []
-        for category in sorted(categories.getKeys()):
+        for category in sorted(categories.keys()):
             if category in self.request:
                 weight = self.request[category]
             else:
@@ -399,13 +673,17 @@ class WeightCategoriesView(object):
                             weight = weight[:-1]
             row = {
                 'category': category,
-                'category_value': categories.getValue(category, language),
+                'category_value': categories[category],
                 'weight': weight,
                 }
             result.append(row)
         return result
 
 
+class FlourishWeightCategoriesView(WeightCategoriesView, flourish.page.Page):
+    """A flourish view for providing category weights for the worksheet."""
+
+
 class ExternalActivitiesTerms(object):
     """Terms for external activities"""
 
@@ -523,18 +801,17 @@ class LinkedColumnBase(BrowserView):
             return self.context.label
 
     def getCategories(self):
-        language = 'en' # XXX this need to be dynamic
-        categories = getCategories(ISchoolToolApplication(None))
+        categories = interfaces.ICategoryContainer(ISchoolToolApplication(None))
 
         results = []
         if self.addForm:
             default_category = defaultCategory(None)
         else:
             default_category = self.context.category
-        for category in sorted(categories.getKeys()):
+        for category in sorted(categories.keys()):
             result = {
                 'name': category,
-                'value': categories.getValue(category, language),
+                'value': categories[category],
                 'selected': category == default_category and 'selected' or None,
                 }
             results.append(result)
@@ -552,7 +829,7 @@ class LinkedColumnBase(BrowserView):
         for term in sorted(term_dict.keys(), key=lambda t: t.first):
             term_disp = term.title
             for section_index, section in enumerate(term_dict[term]):
-                section_disp = list(section.courses)[0].title
+                section_disp = section.title
                 worksheets = interfaces.IActivities(section).values()
                 worksheets = [worksheet for worksheet in worksheets
                               if not worksheet.deployed
@@ -586,10 +863,11 @@ class LinkedColumnBase(BrowserView):
     def getRequestSource(self):
         for key in self.request:
             parts = key.split('_')
-            if len(parts) == 4:
+            if len(parts) > 3:
                 try:
-                    int(parts[2])
-                    return key
+                    sourceObj = getSourceObj(key)
+                    if sourceObj is not None:
+                        return key
                 except:
                     pass
         return None
@@ -637,6 +915,19 @@ class AddLinkedColumnView(LinkedColumnBase):
             self.request.response.redirect(self.nextURL())
 
 
+class FlourishLinkedColumnAddView(flourish.page.Page, AddLinkedColumnView):
+    """flourish view for adding a linked column to the gradebook"""
+
+    def object_title(self):
+        return LinkedColumnBase.title(self)
+
+    def object_label(self):
+        return LinkedColumnBase.label(self)
+
+    def update(self):
+        AddLinkedColumnView.update(self)
+
+
 class EditLinkedColumnView(LinkedColumnBase):
     """View for editing a linked column in the gradebook"""
 
@@ -673,9 +964,25 @@ class EditLinkedColumnView(LinkedColumnBase):
             self.request.response.redirect(self.nextURL())
 
 
+class FlourishEditLinkedColumnView(flourish.page.Page, EditLinkedColumnView):
+    """flourish view for adding a linked column to the gradebook"""
+
+    def object_title(self):
+        return LinkedColumnBase.title(self)
+
+    def object_label(self):
+        return LinkedColumnBase.label(self)
+
+    def actionURL(self):
+        return absoluteURL(self.context, self.request) + '/edit.html'
+
+    def update(self):
+        EditLinkedColumnView.update(self)
+
+
 def defaultCategory(adapter):
-    categories = getCategories(ISchoolToolApplication(None))
-    return categories.getDefaultKey()
+    categories = interfaces.ICategoryContainer(ISchoolToolApplication(None))
+    return categories.default_key
 
 
 ActivityDefaultCategory = widget.ComputedWidgetAttribute(
@@ -684,6 +991,12 @@ ActivityDefaultCategory = widget.ComputedWidgetAttribute(
     )
 
 
+ActivityFormDefaultCategory = widget.ComputedWidgetAttribute(
+    defaultCategory,
+    field=IActivityForm['category']
+    )
+
+
 def defaultDueDate(adapter):
     today = queryUtility(IDateManager).today
     return today
@@ -693,3 +1006,41 @@ ActivityDefaultDueDate = widget.ComputedWidgetAttribute(
     defaultDueDate,
     field=interfaces.IActivity['due_date']
     )
+
+
+ActivityFormDefaultDueDate = widget.ComputedWidgetAttribute(
+    defaultDueDate,
+    field=IActivityForm['due_date']
+    )
+
+
+class ActivityAddTertiaryNavigationManager(flourish.page.TertiaryNavigationManager):
+
+    template = InlineViewPageTemplate("""
+        <ul tal:attributes="class view/list_class">
+          <li tal:repeat="item view/items"
+              tal:attributes="class item/class"
+              tal:content="structure item/viewlet">
+          </li>
+        </ul>
+    """)
+
+    @property
+    def items(self):
+        result = []
+        path = self.request['PATH_INFO']
+        current = path[path.rfind('/')+1:]
+        actions = [
+            ('addActivity.html', _('Activity')),
+            ('addLinkedColumn.html', _('Linked Column')),
+            ('addLinkedActivity.html', _('External Score')),
+            ]
+        for action, title in actions:
+            url = '%s/%s' % (absoluteURL(self.context, self.request), action)
+            title = translate(title, context=self.request)
+            result.append({
+                'class': action == current and 'active' or None,
+                'viewlet': u'<a href="%s">%s</a>' % (url, title),
+                })
+        return result
+
