@@ -24,8 +24,6 @@ $Id$
 import unittest, doctest
 from pprint import pprint
 
-import z3c.optionstorage
-import zope.annotation.interfaces
 from zope.app.testing import setup
 from zope.component import provideAdapter, provideUtility
 from zope.interface import classImplements
@@ -37,6 +35,7 @@ from schooltool.requirement import testing
 from schooltool.requirement.interfaces import IHaveEvaluations
 from schooltool.term.interfaces import IDateManager
 from schooltool.gradebook import activity, gradebook, interfaces
+from schooltool.gradebook import category
 from schooltool.gradebook.tests import stubs
 
 
@@ -46,17 +45,14 @@ def setUp(test):
     testing.setUpEvaluation()
     testing.fixDecimal()
 
-    provideAdapter(
-        z3c.optionstorage.OptionStorage,
-        (zope.annotation.interfaces.IAnnotatable,),
-        z3c.optionstorage.interfaces.IOptionStorage)
-
     provideAdapter(
         activity.getSectionActivities,
         (ISection,), interfaces.IActivities)
 
     provideAdapter(gradebook.Gradebook)
 
+    provideAdapter(category.getCategories)
+
     provideAdapter(
         stubs.SomeProductStub,
         (ISection,), interfaces.IExternalActivities,
